// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.                                        *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber, G. Lo Presti and L. Orsini                      *
 *                                                                       *
 * For the licensing terms see LICENSE.                                  *
 * For the list of contributors see CREDITS.                             *
 *************************************************************************/

#include <string>

#include "xupnp/Platform.h"

/*! This programs creates and searches all supported
    advertisement types.
    This program uses the xupnp classes directly.
 */

int main (int argc, char**argv)
{
	try
	{
		xplore::xupnp::Platform* p = xplore::xupnp::Platform::getInstance();
		xplore::PeerGroup* g = p->getNetPeerGroup();
		xplore::DiscoveryService* ds = g->getDiscoveryService();

		xplore::Advertisement::Reference adv1 = p->createAdvertisement("Peer");
		adv1->setProperty("svc", "http://testmachine.org");
		adv1->setProperty("name", "TestApplication1");

		xplore::Advertisement::Reference adv2 = p->createAdvertisement("PeerGroup");
		adv2->setProperty("name", "TestGroup");

		xplore::Advertisement::Reference adv3 = p->createAdvertisement("Endpoint");
		adv3->setProperty("svc", "http://testmachine.org");
		adv3->setProperty("address", "tcp://mycomputer.org:50000");

		xplore::Advertisement::Reference adv4 = p->createAdvertisement("Peer");
		adv4->setProperty("svc", "http://testmachine.org");
		adv4->setProperty("name", "TestApplication2");

		// Publish advertisement
		ds->publishAdvertisement(adv1);
		ds->publishAdvertisement(adv2);
		ds->publishAdvertisement(adv3);
		ds->publishAdvertisement(adv4);

		// See if they have been cached properly
		std::vector<std::string> a;
		a = ds->getKnownAdvertisementTypes();
		for (unsigned int i = 0; i < a.size(); i++)
		{
			std::cout << "Cached types after search: " << a[i] << std::endl;
		}

		std::cout << std::endl;

		// Remove advertisements by type
		ds->flushAdvertisement(adv4);

		// See if they have been cached properly
		std::vector<std::string> a2;
		a2 = ds->getKnownAdvertisementTypes();
		for (unsigned int i = 0; i < a2.size(); i++)
		{
			std::cout << "Cached after flush: " << a2[i] << std::endl;
		}

		std::cout << std::endl;

		ds->flushAdvertisement(adv1);

		// See if they have been cached properly
		std::vector<std::string> a3;
		a3 = ds->getKnownAdvertisementTypes();
		for (unsigned int i = 0; i < a3.size(); i++)
		{
			std::cout << "Cached after flush: " << a3[i] << std::endl;
		}

		std::cout << std::endl;

		ds->flushAdvertisements("PeerGroup");
		ds->flushAdvertisements("Endpoint");

		// See if they have been cached properly
		std::vector<std::string> a4;
		a4 = ds->getKnownAdvertisementTypes();
		for (unsigned int i = 0; i < a4.size(); i++)
		{
			std::cout << "Cached after flush: " << a4[i] << std::endl;
		}

		std::cout << std::endl;

		xplore::xupnp::Platform::destroyInstance();
	}
	catch (xplore::exception::Exception& e)
	{
		std::cout << "Caught: " << e.message() << std::endl;
	}
	return 0;
}
